#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Help tests"""
import re

from click.testing import CliRunner
from c8ylp.main import cli
from tests.env import Environment


def test_show_version(env: Environment):
    """Test show version"""
    runner = CliRunner()
    result = runner.invoke(cli, ["version"], env=env.create_empty_env())
    assert re.match(r"Version \S+\n", result.output) is not None
